import flet as ft
from Components import Components
from DataManager import DataManager


class ChatTab(ft.Tab):
    def __init__(self, content:ft.Control=None, tab_content:ft.Control=None) -> None:
        super().__init__()
        self.page: ft.Page = Components.get("page")
        self.tab_content = ft.Text("Chat")
        self.__dataManager = DataManager()
        self.__dataManager.load()

        self.txtUser:ft.TextField = ft.TextField(
            label="Username", on_submit=self.__setUsername, expand=False,
            border_color="#FFFFFF"
        )
        self.lblUser:ft.Text = ft.Text(visible=False)
        self.lblRoom:ft.Text = ft.Text("None.", weight=ft.FontWeight.BOLD, color=ft.colors.RED_800)

        self.__alertMsg = None

        Components.add("lblRoom", self.lblRoom)
        Components.add("topicMsg_method", self.topicMsg)


        self.userBar = ft.Container(
            content=ft.Column(
                [
                    ft.Divider(15),
                    ft.Container(
                        ft.Row(
                            [
                                ft.Text("Operating as:"),
                                self.lblUser,
                                self.txtUser
                            ]
                        ),
                        on_click=self.userContainer_onClick
                        
                    ),
                    ft.Divider(15, 2, "#555555"),
                    ft.Row(
                        [
                            ft.Text("Current Room:"),
                            self.lblRoom
                        ]
                    ),
                    ft.Divider(15, 2, "#555555"),
                    # self.txtUser
                ]
            )
        )

        # where all the messages will display
        self.chatAreaColumn = ft.Column(
            scroll=ft.ScrollMode.AUTO, auto_scroll=True, height=250
            )
        
        # Components.add("updateConversation", self.updateConversation)
        
        self.chatArea = ft.Container(
            self.chatAreaColumn,
            width=self.page.width
            )

        self.txtMsg = ft.TextField(label="Type something.", on_submit=self.txtMsg_OnSubmit)

        self.btnSend = ft.ElevatedButton(
            text="Send", icon=ft.icons.SEND, on_click=self.btnSend_OnClick
            )

        self.btnAttachment = ft.IconButton(ft.icons.ATTACH_FILE)

        self.typeBarColumn = ft.Column(
            [
                self.txtMsg,
                ft.Row(
                    [
                        self.btnSend,
                        self.btnAttachment
                    ]
                )
            ]
        )
        self.typeBar = ft.Container(self.typeBarColumn)

        self.__body = ft.Column(
            [
                self.userBar,
                self.chatArea,
                self.typeBar
            ]
        )

        self.content = self.__body

        self.__dataManager.onRoomChange.connect(self.dataManager_onRoomChange)

        # for i in range(5):
        #     self.addMsg(f"Message number {i}. Very long waoooooasdojoasd sdjads", "House", "Jsx")

    def dataManager_onRoomChange(self) -> None:
        # print(self.__dataManager.data["chat"])
        # if not chat:
        #     return
        
        print("Changing Room...")
        # self.chatAreaColumn.clean()
        self.__refreshChat()
        return
        
    
    def __refreshChat(self) -> None:
        # print(self.lblRoom.value)
        if not self.lblRoom.value or self.lblRoom.value == "None.":
            print("Cant refresh chat.")
            return
        
        chat: list = self.__dataManager.data["chat"][self.lblRoom.value]

        if not chat:
            return
        
        self.chatAreaColumn.clean()
        for msgDict in chat:
            user: str = msgDict["user"]
            msg: str = msgDict["text"]
            # user = msg.split(" ::")[0].replace(" says", "")
            # print(f"'{user}'")
            # print(f"'{self.lblUser.value}'")
            # m = msg.split(":: ", 1)[1]
            self.addMsg(msg, self.lblRoom.value, user)
        
        return

    def userContainer_onClick(self, e) -> None:
        if self.txtUser.visible:
            return
        
        self.lblUser.visible = False
        self.txtUser.visible = True
        self.page.update()
        return

    def __setUsername(self, e: ft.ControlEvent) -> None:
        if self.txtUser.value:
            self.lblUser.value = self.txtUser.value
            self.txtUser.visible = False
            self.lblUser.visible = True

            if not self.lblRoom.value == "None.":
                self.__refreshChat()

            self.page.update()
            return
        return

    def __closeAlertDialog(self, e: ft.ControlEvent) -> None:
        self.page.close(self.__alertMsg)
        self.__alertMsg = None
        return

    def __openAlertDialog(self, title:str, msg:str) -> None:
        self.__alertMsg = ft.AlertDialog(
            modal=False, 
            title=ft.Text(title),
            content=ft.Container(
                ft.Text(msg),
                expand=False
                # height=60
            ),
            actions=[
                ft.ElevatedButton(
                    "Okay", on_click=self.__closeAlertDialog
                )
            ],
        )
        self.page.open(self.__alertMsg)
        return

    def build(self) -> "ChatTab":
        return self
    
    def validateUser(self) -> bool:
        if self.lblUser.value:
            return True
        
        self.__openAlertDialog("Username required!", "You need to set a username first.")
        self.txtUser.focus()
        return False
    
    def validateRoom(self) -> bool:
        if self.lblRoom.value and not self.lblRoom.value == "None.":
            return True
        
        self.__openAlertDialog("Room Required!", "You need to select a room first.")
        tc: ft.Tabs = Components.get("tabControl")
        tc.selected_index = 1
        self.page.update()
        return False
    
    def addMsg(self, msg: str, topic: str, user:str=None) -> None:
        _user = user or self.lblUser.value
        isYou = user == self.lblUser.value
        _alignment = {False: "left", True: "right"}
        text = f"{_user} says: {msg}"
        # prefix = 

        msgBubble = ft.Container(
  
                    ft.Column(
                        [
                            ft.Text(
                                "[ YOU ]",
                                size=15,
                                weight="bold",
                                visible=isYou,
                                text_align="left"
                            ),
                            # ft.Text(
                            #     f"Room: {topic}",
                            #     size=15,
                            #     text_align="left",
                            # ),
                            ft.Text(
                                f"{user}",
                                size=15,
                                text_align="left",
                                weight="bold",
                                visible= not isYou # Hide it when is you. Show it when not.
                            ),
                            ft.Text(
                                msg,
                                # text_align=_alignment[isYou],
                                no_wrap=False,
                                width=self.chatArea.width
                                # expand=True
                            )
                        ],
                        # alignment=ft.alignment.center_right,

                        # expand=True
                    ),
            bgcolor="#42554A" if isYou else "#465155",
            padding=ft.Padding(10, 5, 10, 5),
            border_radius=ft.border_radius.all(5.0),
            # alignment=ft.alignment.center_right,
            
            
            width=self.chatArea.width,
            # expand=True
        )
        self.chatAreaColumn.controls.append(msgBubble)
        self.page.update()
            
        return
    
    def sendMsg(self, msg: str=None) -> None:
        self.page.pubsub.send_all_on_topic(
            self.lblRoom.value, f"{self.lblUser.value}]]]{msg}")
        return
    
    def validateSaveAndSend(self) -> None:
        if self.txtMsg.value and self.validateUser() and self.validateRoom():
            # self.addMsg(self.txtMsg.value, self.lblRoom.value, self.lblUser.value)
            self.sendMsg(self.txtMsg.value)
            # print(self.__dataManager.data["chat"][self.lblRoom.value])
            # self.__dataManager.data["chat"][self.lblRoom.value].append(
            #     {self.lblUser.value:  self.txtMsg.value}
            # )

            # self.__dataManager.data["chat"][self.lblRoom.value].append(
            #     f"{self.lblUser.value} says :: {self.txtMsg.value}"
            # )

            self.__dataManager.data["chat"][self.lblRoom.value].append(
                {"user": self.lblUser.value, "text": self.txtMsg.value}
            )
            self.__dataManager.save()
            self.txtMsg.value = ""
            self.page.update()
            return
    
    def btnSend_OnClick(self, e) -> None:
        self.validateSaveAndSend()
        return

    def topicMsg(self, topic: str, msg: str) -> None:
        # print("Message received.")
        user = msg.split("]]]", 1)[0]
        msg = msg.split("]]]", 1)[1]
        self.addMsg(msg, topic, user)
        return
    
    def txtMsg_OnSubmit(self, e: ft.ControlEvent=None) -> None:
        self.validateSaveAndSend()
        # if self.validateUser() and self.validateRoom():
        #     self.addMsg(self.txtMsg.value, self.lblRoom.value, self.lblUser.value)
        #     self.sendMsg(self.txtMsg.value)
        return


